from dataclasses import dataclass, field
from typing import Optional

from dataclasses_json import config as json_config
from dataclasses_json import dataclass_json

from nldcsc.generic.utils import exclude_optional_dict
from nldcsc.plugins.vuln_tracker.objects.data_class_validations import Validations


@dataclass_json
@dataclass
class Severity(Validations):
    type: str
    score: str


@dataclass_json
@dataclass
class Package(Validations):
    ecosystem: str
    name: str

    purl: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )


@dataclass_json
@dataclass
class Event(Validations):
    introduced: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    fixed: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    last_affected: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    limit: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )


@dataclass_json
@dataclass
class Range(Validations):
    type: str
    events: list[Event]

    repo: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    database_specific: Optional[dict] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )

    def __post_init__(self):
        if isinstance(self.events, list):
            self.events = [Event(**dict(x)) for x in self.events]


@dataclass_json
@dataclass
class Affected(Validations):
    package: Optional[Package] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    severity: Optional[Severity] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    ranges: Optional[list[Range]] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    versions: Optional[list[str]] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    ecosystem_specific: Optional[dict] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    database_specific: Optional[dict] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )

    def __post_init__(self):
        if isinstance(self.package, dict):
            self.package = Package(**self.package)

        if isinstance(self.severity, dict):
            self.severity = Severity(**self.severity)

        if hasattr(self, "ranges"):
            if isinstance(self.ranges, list):
                self.ranges = [Range(**dict(x)) for x in self.ranges]


@dataclass_json
@dataclass
class Reference(Validations):
    type: str
    url: str


@dataclass_json
@dataclass
class Credit(Validations):
    name: str

    type: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    contact: Optional[list[str]] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )


@dataclass_json
@dataclass
class OSV(Validations):
    id: str
    modified: str

    schema_version: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    published: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    withdrawn: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    aliases: Optional[list[str]] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    related: Optional[list[str]] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    upstream: Optional[list[str]] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    summary: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    details: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    severity: Optional[list[Severity]] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    affected: Optional[list[Affected]] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    references: Optional[list[Reference]] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    credits: Optional[list[Credit]] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    database_specific: Optional[dict] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )

    def __post_init__(self):
        if hasattr(self, "severity"):
            if isinstance(self.severity, list):
                self.severity = [Severity(**dict(x)) for x in self.severity]

        if hasattr(self, "affected"):
            if isinstance(self.affected, list):
                self.affected = [Affected(**dict(x)) for x in self.affected]

        if hasattr(self, "references"):
            if isinstance(self.references, list):
                self.references = [Reference(**dict(x)) for x in self.references]

        if hasattr(self, "credits"):
            if isinstance(self.credits, list):
                self.credits = [Credit(**dict(x)) for x in self.credits]

    @property
    def cve_id(self) -> str:
        return f"{self.id}"

    @property
    def publisher(self) -> str:
        return self.id.split("-")[0]

    def __repr__(self) -> str:
        return f"OSV({self.publisher}[{self.id}])"



from datetime import datetime


def timestamp_to_strf_string(timestamp: int) -> str:
    """
    Method that will take the provided timestamp and converts it into a RFC3339 date time string

    Args:
        timestamp: unix timestamp

    Returns:
        Date time string
    """
    value = datetime.utcfromtimestamp(timestamp)

    return value.strftime("%Y-%m-%d %H:%M:%S")



import time
from datetime import datetime

import colors
import rfc3339
from flask import g, request, Flask, Response

from nldcsc.flask_midddleware.base import BaseHTTPMiddleware


class DebugLoggingMiddleware(BaseHTTPMiddleware):
    def __init__(self, app: Flask):
        super().__init__(app=app)

    def _before_request(self):
        g.start = time.time()

    def _after_request(self, response: Response):
        now = time.time()
        duration = round(now - g.start, 2)
        dt = datetime.fromtimestamp(now)
        timestamp = rfc3339.rfc3339(dt, utc=True)

        ip = request.headers.get("X-Forwarded-For", request.remote_addr)
        host = request.host.split(":", 1)[0]
        args = dict(request.args)

        log_params = [
            ("method", request.method, "blue"),
            ("path", request.path, "blue"),
            ("status", response.status, "yellow"),
            ("duration", duration, "green"),
            ("time", timestamp, "magenta"),
            ("ip", ip, "gray"),
            ("host", host, "gray"),
            ("params", args, "blue"),
        ]

        request_id = request.headers.get("X-Request-ID")
        if request_id:
            log_params.append(("request_id", request_id, "yellow"))

        parts = []
        for name, value, color in log_params:
            part = colors.color("{}={}".format(name, value), fg=color)
            parts.append(part)
        line = " ".join(parts)

        self.logger.info(line)

        return response



from dataclasses import dataclass, field
from typing import Optional

from dataclasses_json import config as json_config
from dataclasses_json import dataclass_json

from nldcsc.generic.utils import exclude_optional_dict
from nldcsc.plugins.vuln_tracker.objects.common import CVSS2, CVSS30, CVSS31
from nldcsc.plugins.vuln_tracker.objects.data_class_validations import Validations


@dataclass_json
@dataclass
class Publisher(Validations):
    category: str
    name: str
    namespace: str

    contact_details: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    issuing_authority: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )


@dataclass_json
@dataclass
class Revision(Validations):
    date: str
    number: str
    summary: str

    legacy_version: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )


@dataclass_json
@dataclass
class Engine(Validations):
    name: str

    version: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )


@dataclass_json
@dataclass
class Generator(Validations):
    engine: Engine

    date: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )

    def __post_init__(self):
        if isinstance(self.engine, dict):
            self.engine = Engine(**self.engine)


@dataclass_json
@dataclass
class Tracking(Validations):
    current_release_date: str
    id: str
    initial_release_date: str
    revision_history: list[Revision]
    status: str
    version: str

    aliases: Optional[list[str]] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    generator: Optional[Generator] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )

    def __post_init__(self):
        if isinstance(self.revision_history, list):
            self.revision_history = [Revision(**dict(x)) for x in self.revision_history]

        if hasattr(self, "generator"):
            if isinstance(self.generator, dict):
                self.generator = Generator(**self.generator)


@dataclass_json
@dataclass
class Acknowledgment(Validations):
    summary: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    organization: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    names: Optional[list[str]] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    urls: Optional[list[str]] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )


@dataclass_json
@dataclass
class AggregateSeverity(Validations):
    text: str

    namespace: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )


@dataclass_json
@dataclass
class TLP(Validations):
    label: str

    url: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )


@dataclass_json
@dataclass
class Distribution(Validations):
    text: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    tlp: Optional[TLP] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )

    def __post_init__(self):
        if hasattr(self, "tlp"):
            if isinstance(self.tlp, dict):
                self.tlp = TLP(**self.tlp)


@dataclass_json
@dataclass
class Note(Validations):
    category: str
    text: str

    title: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    audience: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )


@dataclass_json
@dataclass
class Reference(Validations):
    summary: str
    url: str

    category: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )


@dataclass_json
@dataclass
class Document(Validations):
    """Captures the meta-data about this document describing a particular set of security advisories."""

    category: str
    csaf_version: str
    publisher: Publisher
    title: str
    tracking: Tracking

    acknowledgments: Optional[list[Acknowledgment]] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    aggregate_severity: Optional[AggregateSeverity] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    distribution: Optional[Distribution] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    lang: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    notes: Optional[list[Note]] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    references: Optional[list[Reference]] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    source_lang: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )

    def __post_init__(self):
        if isinstance(self.publisher, dict):
            self.publisher = Publisher(**self.publisher)

        if isinstance(self.tracking, dict):
            self.tracking = Tracking(**self.tracking)

        if hasattr(self, "acknowledgments"):
            if isinstance(self.acknowledgments, list):
                self.acknowledgments = [
                    Acknowledgment(**dict(x)) for x in self.acknowledgments
                ]

        if hasattr(self, "aggregate_severity"):
            if isinstance(self.aggregate_severity, dict):
                self.aggregate_severity = AggregateSeverity(**self.aggregate_severity)

        if hasattr(self, "distribution"):
            if isinstance(self.distribution, dict):
                self.distribution = Distribution(**self.distribution)

        if hasattr(self, "notes"):
            if isinstance(self.notes, list):
                self.notes = [Note(**dict(x)) for x in self.notes]

        if hasattr(self, "references"):
            if isinstance(self.references, list):
                self.references = [Reference(**dict(x)) for x in self.references]


@dataclass_json
@dataclass
class FileHash(Validations):
    algorithm: str
    value: str


@dataclass_json
@dataclass
class Hashes(Validations):
    filename: str
    file_hashes: list[FileHash]

    def __post_init__(self):
        if isinstance(self.file_hashes, list):
            self.file_hashes = [FileHash(**dict(x)) for x in self.file_hashes]


@dataclass_json
@dataclass
class GenericUri(Validations):
    namespace: str
    uri: str


@dataclass_json
@dataclass
class ProductIdHelper(Validations):
    cpe: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    hashes: Optional[list[Hashes]] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    model_numbers: Optional[list[str]] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    purl: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    sbom_urls: Optional[list[str]] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    serial_numbers: Optional[list[str]] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    skus: Optional[list[str]] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    x_generic_uris: Optional[list[GenericUri]] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )

    def __post_init__(self):
        if hasattr(self, "hashes"):
            if isinstance(self.hashes, list):
                self.hashes = [Hashes(**dict(x)) for x in self.hashes]

        if hasattr(self, "x_generic_uris"):
            if isinstance(self.x_generic_uris, list):
                self.x_generic_uris = [
                    GenericUri(**dict(x)) for x in self.x_generic_uris
                ]


@dataclass_json
@dataclass
class FullProductName(Validations):
    name: str
    product_id: str

    product_identification_helper: Optional[ProductIdHelper] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )

    def __post_init__(self):
        if hasattr(self, "product_identification_helper"):
            if isinstance(self.product_identification_helper, dict):
                self.product_identification_helper = ProductIdHelper(
                    **self.product_identification_helper
                )


@dataclass_json
@dataclass
class Branches(Validations):
    category: str
    name: str

    branches: Optional[list["Branches"]] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    product: Optional[FullProductName] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )

    def __post_init__(self):
        if hasattr(self, "branches"):
            if isinstance(self.branches, list):
                self.branches = [Branches(**dict(x)) for x in self.branches]

        if hasattr(self, "product"):
            if isinstance(self.product, dict):
                self.product = FullProductName(**self.product)


@dataclass_json
@dataclass
class ProductGroup(Validations):
    group_id: str
    product_ids: list[str]

    summary: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )


@dataclass_json
@dataclass
class Relationship(Validations):
    category: str
    full_product_name: FullProductName
    product_reference: str
    relates_to_product_reference: str

    def __post_init__(self):
        if isinstance(self.full_product_name, dict):
            self.full_product_name = FullProductName(**self.full_product_name)


@dataclass_json
@dataclass
class ProductTree(Validations):
    branches: Optional[list[Branches]] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    full_product_names: Optional[list[FullProductName]] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    product_groups: Optional[list[ProductGroup]] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    relationships: Optional[list[Relationship]] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )

    def __post_init__(self):
        if hasattr(self, "branches"):
            if isinstance(self.branches, list):
                self.branches = [Branches(**dict(x)) for x in self.branches]

        if hasattr(self, "full_product_names"):
            if isinstance(self.full_product_names, list):
                self.full_product_names = [
                    FullProductName(**dict(x)) for x in self.full_product_names
                ]

        if hasattr(self, "product_groups"):
            if isinstance(self.product_groups, list):
                self.product_groups = [
                    ProductGroup(**dict(x)) for x in self.product_groups
                ]

        if hasattr(self, "relationships"):
            if isinstance(self.relationships, list):
                self.relationships = [
                    Relationship(**dict(x)) for x in self.relationships
                ]


@dataclass_json
@dataclass
class CWE(Validations):
    id: str
    name: str


@dataclass_json
@dataclass
class Flag(Validations):
    label: str

    date: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    group_ids: Optional[list[str]] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    product_ids: Optional[list[str]] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )


@dataclass_json
@dataclass
class ID(Validations):
    system_name: str
    text: str


@dataclass_json
@dataclass
class Involvement(Validations):
    party: str
    status: str

    date: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    summary: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )


@dataclass_json
@dataclass
class ProductStatus(Validations):
    first_affected: Optional[list[str]] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    first_fixed: Optional[list[str]] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    fixed: Optional[list[str]] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    known_affected: Optional[list[str]] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    known_not_affected: Optional[list[str]] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    last_affected: Optional[list[str]] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    recommended: Optional[list[str]] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    under_investigation: Optional[list[str]] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )


@dataclass_json
@dataclass
class RestartRequired(Validations):
    category: str

    details: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )


@dataclass_json
@dataclass
class Remediation(Validations):
    category: str
    details: str

    date: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    entitlements: Optional[list[str]] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    group_ids: Optional[list[str]] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    product_ids: Optional[list[str]] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    restart_required: Optional[RestartRequired] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    url: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )

    def __post_init__(self):
        if hasattr(self, "restart_required"):
            if isinstance(self.restart_required, dict):
                self.restart_required = RestartRequired(**self.restart_required)


@dataclass_json
@dataclass
class Score(Validations):
    products: list[str]

    cvss_v2: Optional[CVSS2] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    cvss_v3: Optional[CVSS30 | CVSS31] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )

    def __post_init__(self):
        if hasattr(self, "cvss_v2"):
            if isinstance(self.cvss_v2, dict):
                self.cvss_v2 = CVSS2(**self.cvss_v2)

        if hasattr(self, "cvss_v3"):
            if isinstance(self.cvss_v3, dict):
                if self.cvss_v3["version"] == "3.0":
                    self.cvss_v3 = CVSS30(**self.cvss_v3)
                else:
                    self.cvss_v3 = CVSS31(**self.cvss_v3)


@dataclass_json
@dataclass
class Threat(Validations):
    category: str
    details: str

    date: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    group_ids: Optional[list[str]] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    product_ids: Optional[list[str]] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )


@dataclass_json
@dataclass
class Vulnerability(Validations):
    acknowledgments: Optional[list[Acknowledgment]] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    cve: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    cwe: Optional[CWE] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    discovery_date: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    flags: Optional[list[Flag]] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    ids: Optional[list[ID]] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    involvements: Optional[list[Involvement]] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    notes: Optional[list[Note]] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    product_status: Optional[ProductStatus] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    references: Optional[list[Reference]] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    release_date: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    remediations: Optional[list[Remediation]] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    scores: Optional[list[Score]] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    threats: Optional[list[Threat]] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    title: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )

    def __post_init__(self):
        if hasattr(self, "acknowledgments"):
            if isinstance(self.acknowledgments, list):
                self.acknowledgments = [
                    Acknowledgment(**dict(x)) for x in self.acknowledgments
                ]

        if isinstance(self.cwe, dict):
            self.cwe = CWE(**self.cwe)

        if hasattr(self, "flags"):
            if isinstance(self.flags, list):
                self.flags = [Flag(**dict(x)) for x in self.flags]

        if hasattr(self, "ids"):
            if isinstance(self.ids, list):
                self.ids = [ID(**dict(x)) for x in self.ids]

        if hasattr(self, "involvements"):
            if isinstance(self.involvements, list):
                self.involvements = [Involvement(**dict(x)) for x in self.involvements]

        if hasattr(self, "notes"):
            if isinstance(self.notes, list):
                self.notes = [Note(**dict(x)) for x in self.notes]

        if isinstance(self.product_status, dict):
            self.product_status = ProductStatus(**self.product_status)

        if hasattr(self, "references"):
            if isinstance(self.references, list):
                self.references = [Reference(**dict(x)) for x in self.references]

        if hasattr(self, "remediations"):
            if isinstance(self.remediations, list):
                self.remediations = [Remediation(**dict(x)) for x in self.remediations]

        if hasattr(self, "scores"):
            if isinstance(self.scores, list):
                self.scores = [Score(**dict(x)) for x in self.scores]

        if hasattr(self, "threats"):
            if isinstance(self.threats, list):
                self.threats = [Threat(**dict(x)) for x in self.threats]


@dataclass_json
@dataclass
class CSAF(Validations):
    """i.a.w https://docs.oasis-open.org/csaf/csaf/v2.0/csaf_json_schema.json"""

    document: Document

    product_tree: Optional[ProductTree] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    vulnerabilities: Optional[list[Vulnerability]] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )

    def __post_init__(self):
        if isinstance(self.document, dict):
            self.document = Document(**self.document)

        if hasattr(self, "product_tree"):
            if isinstance(self.product_tree, dict):
                self.product_tree = ProductTree(**self.product_tree)

        if hasattr(self, "vulnerabilities"):
            if isinstance(self.vulnerabilities, list):
                self.vulnerabilities = [
                    Vulnerability(**dict(x)) for x in self.vulnerabilities
                ]

    @property
    def cve_id(self) -> str:
        return f"{self.document.tracking.id}"

    @property
    def publisher(self) -> str:
        return f"{self.document.publisher.name}"

    def __repr__(self):
        return f"CSAF({self.document.publisher.name}[{self.document.tracking.id}])"



import os

from eswrap import EsWrap

from nldcsc.generic.utils import getenv_list, getenv_dict


class FlaskEsWrap(object):
    def __init__(self, app=None, init_standalone: bool = False, **kwargs):
        if app and init_standalone:
            raise Exception("App must be None when 'ignore_app_init' is set to True")

        self._es = None
        self.kwargs = kwargs

        self.elastic_user = os.getenv("ELASTIC_USER", None)
        self.elastic_password = os.getenv("ELASTIC_PASSWORD", None)

        # Single Elastic connection details
        self.elastic_host = os.getenv("ELASTIC_HOST", "localhost")
        self.elastic_port = int(os.getenv("ELASTIC_PORT", 9200))
        self.elastic_scheme = os.getenv("ELASTIC_SCHEME", "http")

        # Or as a list[str] | list[dict], takes preference over Single elastic connection details settings
        # When using this option Authentication should be a part of the connection strings,
        # e.g. http://elastic:changeme@localhost:9200/
        self.elastic_connection_details = getenv_list(
            "ELASTIC_CONNECTION_DETAILS", None
        )
        self.elastic_kwargs = getenv_dict("ELASTIC_KWARGS", None)

        if app is not None or init_standalone:
            self.init_app(app)

    def init_app(self, app, **kwargs):
        self.kwargs.update(kwargs)

        if self.elastic_kwargs is not None:
            self.kwargs.update(self.elastic_kwargs)

        if len(self.elastic_connection_details) != 0:
            if self.elastic_user is not None and self.elastic_password is not None:
                self._es = EsWrap(
                    connection_details=self.elastic_connection_details,
                    http_auth=(self.elastic_user, self.elastic_password),
                    **self.kwargs if self.kwargs is not None else {},
                )
            else:
                self._es = EsWrap(
                    connection_details=self.elastic_connection_details,
                    **self.kwargs if self.kwargs is not None else {},
                )
        else:
            if self.elastic_user is not None and self.elastic_password is not None:
                self._es = EsWrap(
                    host=self.elastic_host,
                    port=self.elastic_port,
                    scheme=self.elastic_scheme,
                    http_auth=(
                        self.elastic_user,
                        self.elastic_password,
                    ),
                    **self.kwargs if self.kwargs is not None else {},
                )
            else:
                self._es = EsWrap(
                    host=self.elastic_host,
                    port=self.elastic_port,
                    scheme=self.elastic_scheme,
                    **self.kwargs if self.kwargs is not None else {},
                )

        if app is not None:
            app.flask_eswrap = self

    @property
    def es(self):
        return self._es

    def __del__(self):
        del self._es

    def __repr__(self):
        return f"<< FlaskEsWrap >>"



import logging
import multiprocessing
import os
from pathlib import Path

from flask import Flask
from flask_socketio import SocketIO
from gunicorn.app.base import BaseApplication

from nldcsc.generic.utils import getenv_bool
from nldcsc.loggers.app_logger import AppLogger
from nldcsc.sql_migrations.flask_sql_migrate import FlaskSqlMigrate

logging.setLoggerClass(AppLogger)


class StandaloneApplication(BaseApplication):
    """
    An application interface for configuring a WSGI application to run with gunicorn.
    """

    def __init__(self, app, options=None) -> None:
        self.options = options or {}
        self.application = app
        super().__init__()

    def load_config(self) -> None:
        """
        Method for loading the proper options into the gunicorn configuration.
        """
        config: dict = {
            key: value
            for key, value in self.options.items()
            if key in self.cfg.settings and value is not None
        }
        for key, value in config.items():
            self.cfg.set(key.lower(), value)

    def load(self):
        """
        method that returns the configured WSGI application.
        """
        return self.application


class FlaskAppManager(object):
    """
    An app manager class for configuring and running a flask application
    """

    def __init__(
        self,
        version: str,
        app: Flask,
        socketio: SocketIO = None,
        init_sql_database: bool = False,
        **kwargs,
    ) -> None:
        """
        Initialize a new FlaskAppManager instance.

        Args:
            version: version of the application
            app: Flask application
            socketio: Flask-SocketIO instance
            init_sql_database: Whether to initialize the database
            **kwargs: additional keyword arguments that will be passed to the webserver options dictionary
        """

        self.logger = logging.getLogger(__name__)

        self.logger.info("Initializing FlaskAppManager...")

        self.app = app
        self.socketio = socketio

        if not isinstance(self.app, Flask):
            raise AttributeError(
                f"The provided app variable is not of type 'Flask' but {type(self.app)}"
            )

        self.version: str = version
        self.init_sql_database: bool = init_sql_database

        self.worker_class: str = os.getenv("WEB_WORKER_CLASS", "sync")
        self.max_workers: int = int(os.getenv("WEB_MAX_WORKERS", 0))
        self.web_worker_timeout: int = int(os.getenv("WEB_WORKER_TIMEOUT", 60))

        self.web_tls_key_path: str = os.getenv(
            "WEB_TLS_KEY_PATH", "/app/data/certs/key.pem"
        )
        self.web_tls_cert_path: str = os.getenv(
            "WEB_TLS_CERT_PATH", "/app/data/certs/cert.pem"
        )

        self.app_working_dir: str = os.getenv("APP_WORKING_DIR", "/app")

        self.debug: bool = getenv_bool("DEBUG", "False")
        self.debug_with_ssl: bool = getenv_bool("DEBUG_WITH_SSL", "False")

        self.bind_host: str = os.getenv("BIND_HOST", "localhost")
        self.bind_port: int = int(os.getenv("BIND_PORT", 5050))

        self.kwargs = kwargs

        self.logger.info(
            "Initialization complete, call the run method to start the app!"
        )

    def run(self) -> None:
        """
        Runs the application

        The application will start using gunicorn, if debug is disabled. Otherwise, it will use the flask development
        server.
        """
        self.logger.info("Trying to start the app...")

        try:
            self.logger.info(f"Running version: {self.version}")

            if self.debug:
                if self.debug_with_ssl:
                    self.app.run(
                        host=self.bind_host,
                        port=self.bind_port,
                        ssl_context="adhoc",
                    )
                else:
                    self.app.run(
                        host=self.bind_host,
                        port=self.bind_port,
                    )

            else:
                if self.init_sql_database:
                    if not os.path.exists(
                        os.path.join(self.app_working_dir, "INIT_COMPLETED")
                    ):
                        fsm = FlaskSqlMigrate(
                            cwd=self.app.instance_path.rstrip("/instance"),
                            app_ref=self.app.import_name,
                        )

                        if not os.path.exists(
                            os.path.join(self.app_working_dir, "migrations")
                        ):
                            fsm.db_init()

                        fsm.db_migrate()
                        fsm.db_update()

                        try:
                            Path(
                                os.path.join(self.app_working_dir, "INIT_COMPLETED")
                            ).touch()
                        except FileNotFoundError:
                            os.mkdir(os.path.join(self.app_working_dir))
                            Path(
                                os.path.join(self.app_working_dir, "INIT_COMPLETED")
                            ).touch()
                if self.socketio is None:
                    if self.worker_class not in ["sync", "eventlet", "gevent"]:
                        raise ValueError(
                            f"Unsupported worker_class {self.worker_class} detected!"
                        )

                    options = {
                        "bind": f"{self.bind_host}:{self.bind_port}",
                        "workers": self._number_of_workers(),
                        "timeout": self.web_worker_timeout,
                        "worker_class": self.worker_class,
                        "logger_class": "nldcsc.loggers.gunicorn_logger.GunicornLogger",
                        "access_log_format": "%(t)s src_ip=%(h)s request=%(r)s request_method=%(m)s status=%(s)s "
                        "response_length=%(b)s referrer=%(f)s url=%(U)s query=?%(q)s user_agent=%(a)s t_ms=%(L)s",
                    }

                    options.update(self.kwargs)

                    if os.path.exists(self.web_tls_key_path) and os.path.exists(
                        self.web_tls_cert_path
                    ):
                        options["keyfile"] = self.web_tls_key_path
                        options["certfile"] = self.web_tls_cert_path
                        StandaloneApplication(self.app, options).run()
                    else:
                        # no TLS; assume running behind reverse proxy that handles TLS offloading; switching to
                        # plain http
                        StandaloneApplication(self.app, options).run()
                else:
                    options = {"log_output": True}

                    options.update(self.kwargs)

                    if os.path.exists(self.web_tls_key_path) and os.path.exists(
                        self.web_tls_cert_path
                    ):
                        options["keyfile"] = self.web_tls_key_path
                        options["certfile"] = self.web_tls_cert_path
                        self.socketio.run(
                            app=self.app,
                            host=self.bind_host,
                            port=self.bind_port,
                            **options,
                        )
                    else:
                        # Socketio app
                        # no TLS; assume running behind reverse proxy that handles TLS offloading; switching to
                        # plain http
                        self.socketio.run(
                            app=self.app,
                            host=self.bind_host,
                            port=self.bind_port,
                            **options,
                        )
        except Exception:
            raise

    def _number_of_workers(self) -> int:
        """
        returns number of workers to use.

        number of workers will be based on the configured value or defaults to '(cpu_count * 2) + 1' if nothing is
        configured.

        Returns:
            int: number of workers to use
        """
        if self.max_workers != 0:
            return self.max_workers
        else:
            return (multiprocessing.cpu_count() * 2) + 1



import logging

import requests
from authlib.integrations.base_client.errors import OAuthError
from flask import (
    Blueprint,
    abort,
    current_app,
    flash,
    g,
    redirect,
    request,
    session,
    url_for,
)

from nldcsc.sso.monkey_patch.ssl_verification import ssl_verification

logger = logging.getLogger(__name__)

sso_auth = Blueprint("sso_auth", __name__)


# noinspection PyProtectedMember
@sso_auth.route("/sso_login")
@ssl_verification
def sso_login():
    if current_app.config["SSO_OVERWRITE_REDIRECT_URI"]:
        redirect_uri = current_app.config["SSO_OVERWRITE_REDIRECT_URI"]
    else:
        redirect_uri = url_for("sso_auth.sso_authorize", _external=True)
    if current_app.config["SSO_CALLBACK_ENDPOINT"]:
        session["next"] = (
            f"{request.root_url}{current_app.config['SSO_CALLBACK_ENDPOINT']}"
        )
    else:
        session["next"] = request.args.get("next", request.root_url)
    return g._sso_auth.authorize_redirect(redirect_uri)


# noinspection PyProtectedMember
@sso_auth.route("/sso_authorize")
@ssl_verification
def sso_authorize():
    try:
        token = g._sso_auth.authorize_access_token()
    except OAuthError as e:
        logger.exception("Could not get the access token")
        abort(401, str(e))
    session["sso_auth_token"] = token
    g.sso_id_token = token
    if current_app.config["SSO_USER_INFO_ENABLED"]:
        profile = g._sso_auth.userinfo(token=token)
        session["sso_auth_profile"] = profile
    try:
        return_to = session["next"]
        del session["next"]
    except KeyError:
        return_to = request.root_url
    return redirect(return_to)


# noinspection PyProtectedMember
@sso_auth.route("/sso_logout")
@ssl_verification
def sso_logout():
    """
    Request the browser to please forget the cookie we set, to clear the
    current session.
    """

    tokenResponse = session.get("sso_auth_token")

    if tokenResponse is not None:
        # propagate logout to sso
        endSessionEndpoint = current_app.config["SSO_ENDSESSION_ENDPOINT"]

        try:

            headers = {
                "Authorization": f"Bearer {g._sso_auth.get_access_token()}",
                "Content-Type": "application/x-www-form-urlencoded",
            }

            data = {
                "client_id": current_app.config["SSO_CLIENT_ID"],
                "client_secret": current_app.config["SSO_CLIENT_SECRET"],
                "refresh_token": g._sso_auth.get_refresh_token(),
            }

            with requests.session() as req_session:
                # noinspection RequestsNoVerify
                req_session.post(
                    endSessionEndpoint,
                    data=data,
                    headers=headers,
                    verify=False,
                )
        except AttributeError as err:
            logger.warning(f"Could not get specific attribute: {err}")

    session.pop("sso_auth_token", None)
    session.pop("sso_auth_profile", None)
    g.sso_id_token = None
    reason = request.args.get("reason")
    if reason == "expired":
        flash("Your session expired, please reconnect.")
    else:
        flash("You were successfully logged out.")
    return_to = request.args.get("next", request.root_url)
    return redirect(return_to)



import json
from dataclasses import dataclass, field
from typing import Optional

from dataclasses_json import config as json_config
from dataclasses_json import dataclass_json

from nldcsc.generic.utils import exclude_optional_dict
from nldcsc.plugins.vuln_tracker.objects.common import CVSS40, CVSS31, CVSS30, CVSS2
from nldcsc.plugins.vuln_tracker.objects.data_class_validations import Validations


@dataclass_json
@dataclass
class Reference(Validations):
    url: str

    source: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    tags: Optional[list[str]] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )


@dataclass_json
@dataclass
class LangString(Validations):
    lang: str
    value: str


@dataclass_json
@dataclass
class Cvss40(Validations):
    source: str
    type: str
    cvssData: CVSS40

    def __post_init__(self):
        if isinstance(self.cvssData, dict):
            self.cvssData = CVSS40(**self.cvssData)


@dataclass_json
@dataclass
class Cvss31(Validations):
    source: str
    type: str
    cvssData: CVSS31

    exploitabilityScore: Optional[int] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    impactScore: Optional[int] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )

    def __post_init__(self):
        if isinstance(self.cvssData, dict):
            self.cvssData = CVSS31(**self.cvssData)


@dataclass_json
@dataclass
class Cvss30(Validations):
    source: str
    type: str
    cvssData: CVSS30

    exploitabilityScore: Optional[int] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    impactScore: Optional[int] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )

    def __post_init__(self):
        if isinstance(self.cvssData, dict):
            self.cvssData = CVSS30(**self.cvssData)


@dataclass_json
@dataclass
class Cvss2(Validations):
    source: str
    type: str
    cvssData: CVSS2

    baseSeverity: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    exploitabilityScore: Optional[int] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    impactScore: Optional[int] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    acInsufInfo: Optional[bool] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    obtainAllPrivilege: Optional[bool] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    obtainUserPrivilege: Optional[bool] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    obtainOtherPrivilege: Optional[bool] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    userInteractionRequired: Optional[bool] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )

    def __post_init__(self):
        if isinstance(self.cvssData, dict):
            self.cvssData = CVSS2(**self.cvssData)


@dataclass_json
@dataclass
class Metric(Validations):
    cvssMetricV40: Optional[list[Cvss40]] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    cvssMetricV31: Optional[list[Cvss31]] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    cvssMetricV30: Optional[list[Cvss30]] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    cvssMetricV2: Optional[list[Cvss2]] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )

    def __post_init__(self):
        if hasattr(self, "cvssMetricV40"):
            if isinstance(self.cvssMetricV40, list):
                self.cvssMetricV40 = [Cvss40(**dict(x)) for x in self.cvssMetricV40]

        if hasattr(self, "cvssMetricV31"):
            if isinstance(self.cvssMetricV31, list):
                self.cvssMetricV31 = [Cvss31(**dict(x)) for x in self.cvssMetricV31]

        if hasattr(self, "cvssMetricV30"):
            if isinstance(self.cvssMetricV30, list):
                self.cvssMetricV30 = [Cvss30(**dict(x)) for x in self.cvssMetricV30]

        if hasattr(self, "cvssMetricV2"):
            if isinstance(self.cvssMetricV2, list):
                self.cvssMetricV2 = [Cvss2(**dict(x)) for x in self.cvssMetricV2]


@dataclass_json
@dataclass
class Weakness(Validations):
    source: str
    type: str
    description: list[LangString]

    def __post_init__(self):
        if isinstance(self.description, list):
            self.description = [LangString(**dict(x)) for x in self.description]


@dataclass_json
@dataclass
class CpeMatch(Validations):
    vulnerable: bool
    criteria: str
    matchCriteriaId: str

    versionStartExcluding: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    versionStartIncluding: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    versionEndExcluding: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    versionEndIncluding: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )


@dataclass_json
@dataclass
class Node(Validations):
    operator: str
    cpeMatch: list[CpeMatch]

    negate: Optional[bool] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )

    def __post_init__(self):
        if isinstance(self.cpeMatch, list):
            self.cpeMatch = [CpeMatch(**dict(x)) for x in self.cpeMatch]


@dataclass_json
@dataclass
class NistConfig(Validations):
    nodes: list[Node]

    operator: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    negate: Optional[bool] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )

    def __post_init__(self):
        if isinstance(self.nodes, list):
            self.nodes = [Node(**dict(x)) for x in self.nodes]


@dataclass_json
@dataclass
class VendorComment(Validations):
    organization: str
    comment: str
    lastModified: str


@dataclass_json
@dataclass
class Nist(Validations):
    id: str
    published: str
    lastModified: str
    references: list[Reference]
    descriptions: list[LangString]

    sourceIdentifier: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    vulnStatus: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    evaluatorComment: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    evaluatorSolution: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    evaluatorImpact: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    cisaExploitAdd: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    cisaActionDue: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    cisaRequiredAction: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    cisaVulnerabilityName: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    cveTags: Optional[list[str]] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    metrics: Optional[Metric] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    weaknesses: Optional[list[Weakness]] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    configurations: Optional[list[NistConfig]] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    vendorComments: Optional[list[VendorComment]] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )

    def to_rss_format(self):

        return {
            "id": self.id,
            "published": self.published,
            "lastModified": self.lastModified,
            "descriptions": f"{[x.value for x in self.descriptions if x.lang == 'en']}",
            "references": f"{[x.url for x in self.references]}",
        }

    def __post_init__(self):
        if isinstance(self.references, list):
            self.references = [Reference(**dict(x)) for x in self.references]

        if isinstance(self.descriptions, list):
            self.descriptions = [LangString(**dict(x)) for x in self.descriptions]

        if hasattr(self, "metrics"):
            if isinstance(self.metrics, dict):
                self.metrics = Metric(**self.metrics)

        if hasattr(self, "weaknesses"):
            if isinstance(self.weaknesses, list):
                self.weaknesses = [Weakness(**dict(x)) for x in self.weaknesses]

        if hasattr(self, "configurations"):
            if isinstance(self.configurations, list):
                self.configurations = [
                    NistConfig(**dict(x)) for x in self.configurations
                ]

        if hasattr(self, "vendorComments"):
            if isinstance(self.vendorComments, list):
                self.vendorComments = [
                    VendorComment(**dict(x)) for x in self.vendorComments
                ]

    @property
    def cve_id(self) -> str:
        return f"{self.id}"

    @property
    def publisher(self) -> str:
        return f"{self.sourceIdentifier}"

    def __eq__(self, other: "Nist") -> bool:
        return self.to_dict() == other.to_dict()

    def __lt__(self, other: "Nist") -> bool:
        return self.cve_id < other.cve_id

    def __repr__(self) -> str:
        return f"Nist({self.sourceIdentifier}[{self.id}])"


@dataclass_json
@dataclass
class NistCveHistoryDetail(Validations):
    type: str

    action: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    oldValue: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    newValue: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )


@dataclass_json
@dataclass
class NistCveHistory(Validations):
    cveId: str
    eventName: str
    cveChangeId: str
    sourceIdentifier: str

    created: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    details: Optional[list[NistCveHistoryDetail]] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )

    def __post_init__(self):
        if hasattr(self, "details"):
            if isinstance(self.details, list):
                self.details = [NistCveHistoryDetail(**dict(x)) for x in self.details]

    def __repr__(self) -> str:
        return f"NistCveHistory({self.sourceIdentifier}[{self.cveId}])"


@dataclass_json
@dataclass
class Title(Validations):
    title: str
    lang: str


@dataclass_json
@dataclass
class CpeReference(Validations):
    ref: str

    type: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )


@dataclass_json
@dataclass
class DepCpe(Validations):
    cpeName: str
    cpeNameId: str


@dataclass_json
@dataclass
class NistCpe(Validations):
    deprecated: bool
    cpeName: str
    cpeNameId: str
    created: str
    lastModified: str

    titles: Optional[list[Title]] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    refs: Optional[list[CpeReference]] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    deprecatedBy: Optional[list[DepCpe]] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    deprecates: Optional[list[DepCpe]] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )

    def __post_init__(self):
        if isinstance(self.deprecated, str):
            self.deprecated = json.loads(self.deprecated)

        if hasattr(self, "titles"):
            if isinstance(self.titles, list):
                self.titles = [
                    Title(**dict(x)) for x in self.titles if dict(x)["lang"] == "en"
                ]
            if isinstance(self.titles, str):
                self.titles = [Title(**dict(x)) for x in json.loads(self.titles)]

        if hasattr(self, "refs"):
            if isinstance(self.refs, list):
                self.refs = [CpeReference(**dict(x)) for x in self.refs]
            if isinstance(self.refs, str):
                self.refs = [CpeReference(**dict(x)) for x in json.loads(self.refs)]

        if hasattr(self, "deprecatedBy"):
            if isinstance(self.deprecatedBy, list):
                self.deprecatedBy = [DepCpe(**dict(x)) for x in self.deprecatedBy]
            if isinstance(self.deprecatedBy, str):
                self.deprecatedBy = [
                    DepCpe(**dict(x)) for x in json.loads(self.deprecatedBy)
                ]

        if hasattr(self, "deprecates"):
            if isinstance(self.deprecates, list):
                self.deprecates = [DepCpe(**dict(x)) for x in self.deprecates]
            if isinstance(self.deprecates, str):
                self.deprecates = [
                    DepCpe(**dict(x)) for x in json.loads(self.deprecates)
                ]

    def to_redis(self):
        data = self.to_dict()

        data["deprecated"] = json.dumps(data["deprecated"])

        if "titles" in data:
            data["titles"] = json.dumps(data["titles"])
        if "refs" in data:
            data["refs"] = json.dumps(data["refs"])
        if "deprecatedBy" in data:
            data["deprecatedBy"] = json.dumps(data["deprecatedBy"])
        if "deprecates" in data:
            data["deprecates"] = json.dumps(data["deprecates"])

        return data

    def __repr__(self) -> str:
        if self.deprecated:
            return f"NistCpe({self.cpeName}[{self.cpeNameId}]) **DEPRECATED**"
        else:
            return f"NistCpe({self.cpeName}[{self.cpeNameId}])"


@dataclass_json
@dataclass
class CpeName(Validations):
    cpeName: str
    cpeNameId: str


@dataclass_json
@dataclass
class NistCpeMatch(Validations):
    criteria: str
    matchCriteriaId: str
    created: str
    lastModified: str
    status: str

    versionStartExcluding: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    versionStartIncluding: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    versionEndExcluding: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    versionEndIncluding: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    cpeLastModified: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    matches: Optional[list[CpeName]] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )

    def __post_init__(self):
        if hasattr(self, "matches"):
            if isinstance(self.matches, list):
                self.matches = [CpeName(**dict(x)) for x in self.matches]
            if isinstance(self.matches, str):
                self.matches = [CpeName(**dict(x)) for x in json.loads(self.matches)]

    def __repr__(self) -> str:
        return f"NistCpeMatch({self.criteria}[{self.matchCriteriaId}])"

    def to_redis(self):
        data = self.to_dict()

        if "matches" in data:
            data["matches"] = json.dumps(data["matches"])

        return data



import logging
import os

import mock
from mock.mock import patch

from tests.helpers.capture_logging import catch_logs, records_to_tuples


class TestLogger:
    def test_logger_import(self):
        try:
            from nldcsc.loggers.app_logger import AppLogger

            logging.setLoggerClass(AppLogger)

            logger_name = "TEST"
            logger = logging.getLogger(logger_name)
            logger.propagate = True

            with catch_logs(level=logging.DEBUG, logger=logger) as handler:
                logger.debug("Debug message")
                logger.info("Info message")
                logger.warning("Warning message")
                logger.error("Error message")
                logger.critical("Critical message")
                assert records_to_tuples(handler.records) == [
                    (logger_name, logging.DEBUG, "Debug message"),
                    (logger_name, logging.INFO, "Info message"),
                    (logger_name, logging.WARNING, "Warning message"),
                    (logger_name, logging.ERROR, "Error message"),
                    (logger_name, logging.CRITICAL, "Critical message"),
                ]

        except Exception:
            raise

    def test_handler_types(self):
        os.environ["GELF_SYSLOG"] = "True"

        from nldcsc.loggers.app_logger import AppLogger
        from logging.handlers import RotatingFileHandler
        from nldcsc.loggers.handlers.gelf_handler import DCSCGelfUDPHandler

        logging.setLoggerClass(AppLogger)

        logger = logging.getLogger("test_handler_types")

        assert len(logger.handlers) == 3
        assert isinstance(logger.handlers[0], logging.StreamHandler)
        assert isinstance(logger.handlers[1], RotatingFileHandler)
        assert isinstance(logger.handlers[2], DCSCGelfUDPHandler)

    def test_syslog_handler(self):
        os.environ["GELF_SYSLOG"] = "False"

        from nldcsc.loggers.app_logger import AppLogger
        from logging.handlers import RotatingFileHandler
        from nldcsc.loggers.handlers.syslog_handler import FullSysLogHandler

        logging.setLoggerClass(AppLogger)

        logger = logging.getLogger("test_syslog_handler")

        assert len(logger.handlers) == 3
        assert isinstance(logger.handlers[0], logging.StreamHandler)
        assert isinstance(logger.handlers[1], RotatingFileHandler)
        assert isinstance(logger.handlers[2], FullSysLogHandler)

    @mock.patch("nldcsc.loggers.handlers.syslog_handler.socket.socket.sendto")
    def test_syslog_emit(self, sys_socket):
        os.environ["GELF_SYSLOG"] = "False"
        import socket

        from nldcsc.loggers.app_logger import AppLogger

        logging.setLoggerClass(AppLogger)

        logger = logging.getLogger("test_syslog_emit")
        with patch.object(logger.handlers[2], "socktype", socket.SOCK_DGRAM):
            logger.info("Info test")
            sys_socket.assert_called()

        with patch.object(
            logger.handlers[2], "structured_data", {"data": {"k": "v", "k2": "v2"}}
        ):
            with patch.object(logger.handlers[2], "enterprise_id", 1):
                logger.info("Info structured data")
                sys_socket.assert_called()

        with patch.object(
            logger.handlers[2], "structured_data", {"meta": {"k": "v", "k2": "v2"}}
        ):
            logger.info("Info structured data id")
            sys_socket.assert_called()



from functools import wraps
from pathlib import Path

from nldcsc.http_apis.base_class.api_base_class import ApiBaseClass


def recreate_session(f):
    """Recreate the otrs session if it is no longer valid."""

    @wraps(f)
    def wrapped(*args, **kwargs):

        if args[0].session_id is None:
            args[0].create_session()

        response = f(*args, **kwargs)
        if response.get("Error"):
            # TicketSearch.AuthFail, TicketUpdate.AuthFail, etc
            if ".AuthFail" in response.get("Error", {}).get("ErrorCode"):
                args[0].create_session()
                response = f(*args, **kwargs)
                if response.get("Error"):
                    raise Exception(response)
            else:
                raise Exception(response)
        return response

    return wrapped


class Otrs(ApiBaseClass):
    def __init__(
        self,
        baseurl: str,
        api_path: str,
        auth: tuple = None,
        proxies: dict = None,
        verify: bool | str | None = None,
        user_agent: str = "DevOps",
        save_session_id: bool = False,
        **kwargs,
    ):
        super().__init__(
            baseurl=baseurl,
            api_path=api_path,
            verify=verify,
            proxies=proxies,
            user_agent=user_agent,
            **kwargs,
        )
        self._auth = auth
        self.set_header_field(field="Content-Type", value="application/json")
        self.save_session_id = save_session_id
        self.session_id_path = Path("data/.session_id")
        # If saving is off, then don't use either.
        if self.save_session_id and self.session_id_path.exists():
            self.session_id = self.session_id_path.read_text().strip()
        else:
            self.session_id = None

    def create_session(self):
        resource = "GenericTicketConnectorREST/Session"
        response = self.call(
            method=self.methods.POST,
            resource=resource,
            data={"UserLogin": self._auth[0], "Password": self._auth[1]},
        )
        try:
            self.session_id = response["SessionID"]
            if self.save_session_id:
                self.session_id_path.write_text(self.session_id)
        except Exception:
            raise Exception(f"Error creating session: {response.content}")

    @recreate_session
    def search_ticket(self, **kwargs) -> dict:
        """ "
        TODO: Link docs with all params
        Example params:
        Title='%'+test+'%',
        TicketCreateTimeNewerDate=ts_start,
        SortBy= 'Created',
        OrderBy='Up',
        StateType = ['Open', 'New'],
        # Due to the sorting, If over 25 -> next iteration will catch next batch
        Limit = 25
        """

        resource = "GenericTicketConnectorREST/TicketSearch"

        query_data = {"SessionID": self.session_id}
        query_data.update(**kwargs)
        return self.call(
            method=self.methods.POST,
            resource=resource,
            data=query_data,
        )

    @recreate_session
    def get_ticket(
        self,
        ticket_id: str,
        articles: bool = False,
        attachments: bool = False,
        dynamic_fields: bool = False,
        html_body_as_attachment: bool = False,
    ) -> dict:

        params = {
            "SessionID": self.session_id,
            "AllArticles": int(articles),
            "Attachments": int(attachments),
            "DynamicFields": int(dynamic_fields),
            "HTMLBodyAsAttachment": int(html_body_as_attachment),
        }
        resource = f"GenericTicketConnectorREST/Ticket/{ticket_id}"
        return self.call(
            method=self.methods.GET,
            resource=resource,
            data=params,
        )

    @recreate_session
    def create_ticket(self, ticket: dict, article: dict, dynamic_fields: list = None):
        # https://otrscommunityedition.com/doc/api/otrs/6.0/Perl/Kernel/GenericInterface/Operation/Ticket/TicketCreate.pm.html
        # "Ticket": {
        #    "Title": "test",
        #    "CustomerUser": "test@test.com",
        #    "QueueID": 28,
        #    "Type": "out",
        #    "State": "new",
        #    "Priority": "1 very low",
        # },
        # "Article": {
        #    "Subject": "Test",
        #    "Body": "this is a test",
        #    "ContentType": "text/plain; charset=utf-8",
        # }

        resource = "GenericTicketConnectorREST/Ticket"
        query_data = {
            "SessionID": self.session_id,
            "Ticket": ticket,
            "Article": article,
        }
        if dynamic_fields:
            query_data["DynamicField"] = dynamic_fields

        return self.call(
            method=self.methods.POST,
            resource=resource,
            data=query_data,
        )

    @recreate_session
    def update_ticket(
        self,
        ticket_id: str | None = None,
        ticket_number: int | None = None,
        ticket: dict | None = None,
        article: dict | None = None,
        dynamic_fields: list | None = None,
    ):
        # https://otrscommunityedition.com/doc/api/otrs/6.0/Perl/Kernel/GenericInterface/Operation/Ticket/TicketUpdate.pm.html

        query_data = {"SessionID": self.session_id}

        if ticket_number is not None:
            # The endpoint is not recognized without the /0 but it does nothing.
            resource = "GenericTicketConnectorREST/Ticket/0"
            query_data["TicketNumber"] = ticket_number
        elif ticket_id is not None:
            resource = f"GenericTicketConnectorREST/Ticket/{ticket_id}"
        else:
            raise Exception("MAX one of ticket_id or ticket_number must be provided.")

        if ticket is not None:
            query_data["Ticket"] = ticket
        if article is not None:
            query_data["Article"] = article
        if dynamic_fields is not None:
            query_data["DynamicField"] = dynamic_fields

        return self.call(
            method=self.methods.PATCH,
            resource=resource,
            data=query_data,
        )



import time
from uuid import uuid4
from nldcsc.plugins.redis_client.wrapper import RedisWrapper


class RateLimit:
    def __init__(self, max_attempts: int = 5, time_window: int = 300):
        self.cache = RedisWrapper().redis_client
        self.max_attempts = max_attempts
        self.time_window = time_window

    def get_key(self, id: int):
        return f"{self.__class__.__name__}:limits:{id}"

    def attempt(self, id: int):
        name = self.get_key(id)
        key = uuid4().hex

        self.cache.hset(name, key, int(time.time()))
        self.cache.hexpire(name, self.time_window, key)

    def clear(self, id: int):
        self.cache.unlink(self.get_key(id))

    def limited(self, id: int):
        return self.cache.hlen(self.get_key(id)) >= self.max_attempts



import logging
import os
from contextlib import contextmanager
from typing import Any

import ldap
from ldap.filter import escape_filter_chars

from nldcsc.loggers.app_logger import AppLogger

logging.setLoggerClass(AppLogger)


class LDAPClient(object):
    def __init__(
        self,
        url: str,
        username: str,
        password: str,
        ldap_user_base: str = None,
        ldap_ipa_admin_groups: list = None,
        ldap_ipa_superuser_groups: list = None,
        ldap_ipa_groups: list = None,
        ldap_ca_cert_file: str = None,
        ldap_ca_cert_verify: bool = True,
        ldap_user_template: str = "uid={user_id}",
        ldap_member_field: str = "memberOf",
        ldap_display_name_field: str = "displayName",
        additional_fields: list[str] = None,
    ):
        self._url = url
        self._username = username
        self._password = password

        self.ldap_user_base = ldap_user_base
        self.ldap_ipa_admin_groups = self.convert_to_bytes(ldap_ipa_admin_groups)
        self.ldap_ipa_superuser_groups = self.convert_to_bytes(
            ldap_ipa_superuser_groups
        )
        self.ldap_ipa_groups = (
            self.ldap_ipa_admin_groups
            + self.ldap_ipa_superuser_groups
            + self.convert_to_bytes(ldap_ipa_groups)
        )
        self.ldap_user_template = ldap_user_template
        self.ldap_member_field = ldap_member_field
        self.ldap_display_name_field = ldap_display_name_field
        self.additional_fields = additional_fields or []

        self.ldap_username = f"{self.ldap_user_template.format(user_id=self._username)},{self.ldap_user_base}"
        self.is_admin = None
        self.is_superuser = None

        self.ldap_ca_cert_file = (
            ldap_ca_cert_file
            if ldap_ca_cert_file is not None
            else os.getenv("LDAP_CACERTFILE", None)
        )
        self.ldap_ca_cert_verify = ldap_ca_cert_verify

        self.logger = logging.getLogger(__name__)

    def create_connection(self):
        """Create a connection over LDAP using the provided credentials.

        Raises:
            e: Cuts the connection if an error occurs.

        Returns:
            _type_: An LDAP connection instance.
        """
        try:
            ldap_connection = ldap.initialize(
                self._url,
                bytes_mode=False,
            )
            self.logger.info("Connection to server succeeded...")
        except ldap.SERVER_DOWN:
            raise

        # Set LDAP protocol version used
        ldap_connection.protocol_version = ldap.VERSION3

        if self.ldap_ca_cert_file is not None:
            # Force cert validation
            ldap_connection.set_option(
                ldap.OPT_X_TLS_REQUIRE_CERT, ldap.OPT_X_TLS_DEMAND
            )
            # Set path name of file containing all trusted CA certificates
            ldap_connection.set_option(
                ldap.OPT_X_TLS_CACERTFILE, self.ldap_ca_cert_file
            )
            # Force libldap to create a new SSL context (must be last TLS option!)
            ldap_connection.set_option(ldap.OPT_X_TLS_NEWCTX, 0)
        elif self.ldap_ca_cert_verify is False:
            ldap_connection.set_option(
                ldap.OPT_X_TLS_REQUIRE_CERT, ldap.OPT_X_TLS_NEVER
            )
            ldap_connection.set_option(ldap.OPT_X_TLS_NEWCTX, 0)

        try:
            ldap_connection.set_option(ldap.OPT_REFERRALS, 0)
            ldap_connection.set_option(ldap.OPT_NETWORK_TIMEOUT, 2.0)
            ldap_connection.simple_bind_s(self.ldap_username, self._password)
            self.logger.info("Simple bind successful...")

        except Exception as e:
            ldap_connection.unbind()
            raise e
        return ldap_connection

    @contextmanager
    def ldap_connection(self):
        """Runs the create_connection method with supervision of the context manager.

        Yields:
            _type_: An LDAP connection instance.
        """
        conn = self.create_connection()
        try:
            yield conn
        except Exception as err:
            self.logger.error(f"Error on LDAP contextmanager: {err}")
            self.logger.exception(err)
            raise
        finally:
            conn.unbind()
            self.logger.info("Disconnected from LDAP server!")

    def validate_credentials(self) -> bool:
        try:
            with self.ldap_connection():
                self.logger.info("Connection to LDAP server successful!!!")
                pass
        except ldap.INVALID_CREDENTIALS:
            self.logger.warning(f"Wrong credentials provided... -> {self._username}")
            return False
        except ldap.UNWILLING_TO_PERFORM:
            self.logger.error(
                f"LDAP server is unwilling to bind, possibly locked... -> {self._username}"
            )
            # Too many log in attempts - locked
            return False

        return True

    def validate_user(self) -> tuple[bool, None] | tuple[bool, str | Any]:
        # Fields to return
        FIELDS = [
            self.ldap_member_field,
            self.ldap_display_name_field,
            *self.additional_fields,
        ]

        query = self.ldap_user_template.format(
            user_id=escape_filter_chars(self._username)
        )
        # Get LDAP connection with account
        with self.ldap_connection() as ldap_connection:
            results = ldap_connection.search_s(
                self.ldap_user_base, ldap.SCOPE_SUBTREE, query, attrlist=FIELDS
            )

        if len(results) != 1:
            # Invalid account type or person was deleted between credential check and now.
            self.logger.warning(
                "Invalid account type or person was deleted between credential check and now!"
            )
            return False, None

        fields = results[0][1]

        username = fields.get(self.ldap_display_name_field)

        if username:
            username = username[0].decode("utf-8")
        else:
            # Should always be a username but just in case, return the id
            username = self._username

        if self.additional_fields:
            return_fields = {
                self.ldap_display_name_field: username,
            }

            for field in self.additional_fields:
                field_value = fields.get(field, None)

                if not field_value:
                    continue

                field_value = [f.decode() for f in field_value]

                if len(field_value) == 1:
                    field_value = field_value[0]

            return_fields[field] = field_value
        else:
            return_fields = username
        # Check if user is in a required group
        if any(
            i in fields.get(self.ldap_member_field, []) for i in self.ldap_ipa_groups
        ):
            self.logger.info(f"User: {self._username} granted access; returning")
            self.is_admin = any(
                i in fields.get(self.ldap_member_field, [])
                for i in self.ldap_ipa_admin_groups
            )
            self.is_superuser = any(
                i in fields.get(self.ldap_member_field, [])
                for i in self.ldap_ipa_superuser_groups
            )
            return True, return_fields
        else:
            self.logger.info(f"User: {self._username} denied access; returning")
            return False, None

    @staticmethod
    def convert_to_bytes(groups: list) -> list:
        return [group.encode(encoding="UTF-8", errors="strict") for group in groups]



class Validations:
    def __post_init__(self):
        """Run validation methods if declared.
        The validation method can be a simple check
        that raises ValueError or a transformation to
        the field value.
        The validation is performed by calling a function named:
            `validate_<field_name>(self, value, field) -> field.type`
        """
        for name, field in self.__dataclass_fields__.items():
            if method := getattr(self, f"validate_{name}", None):
                setattr(self, name, method(getattr(self, name), field=field))



from nldcsc.http_apis.base_class.api_base_class import ApiBaseClass


class ProvidentiaApi(ApiBaseClass):
    def __init__(
        self,
        baseurl: str,
        api_path: str = "v3",
        proxies: dict = None,
        user_agent: str = "NLDCSC",
        api_key: str = "",
        **kwargs,
    ):
        super().__init__(
            baseurl=baseurl,
            api_path=api_path,
            proxies=proxies,
            user_agent=user_agent,
            **kwargs,
        )

        self.set_header_field("Authorization", f"Bearer {api_key}")

    def environments(self) -> dict[str, list[dict[str, str]]]:
        resource = ""
        return self.call(self.methods.GET, resource)

    def environment(self, environment: str) -> dict:
        resource = f"/{environment}"
        return self.call(self.methods.GET, resource)

    def environment_hosts(self, environment: str) -> dict:
        resource = f"/{environment}/hosts"
        return self.call(self.methods.GET, resource)

    def environment_hosts_id(self, environment: str, host_id: str) -> dict:
        resource = f"/{environment}/hosts/{host_id}"
        return self.call(self.methods.GET, resource)

    def environment_inventory(self, environment: str) -> dict:
        resource = f"/{environment}/inventory"
        return self.call(self.methods.GET, resource)

    def environment_networks(self, environment: str) -> dict:
        resource = f"/{environment}/networks"
        return self.call(self.methods.GET, resource)

    def environment_tags(self, environment: str) -> dict:
        resource = f"/{environment}/tags"
        return self.call(self.methods.GET, resource)

    def environment_actors(self, environment: str) -> dict:
        resource = f"/{environment}/actors"
        return self.call(self.methods.GET, resource)

    def environment_services(self, environment: str) -> dict:
        resource = f"/{environment}/services"
        return self.call(self.methods.GET, resource)

    def environment_services_id(self, environment: str, service_id: str) -> dict:
        resource = f"/{environment}/services/{service_id}"
        return self.call(self.methods.GET, resource)



from dataclasses import dataclass, field
from typing import Optional

from dataclasses_json import dataclass_json, config as json_config

from nldcsc.generic.utils import exclude_optional_dict
from nldcsc.plugins.vuln_tracker.objects.data_class_validations import Validations


@dataclass_json
@dataclass
class CVSS2(Validations):
    """i.a.w https://www.first.org/cvss/cvss-v2.0.json"""

    version: str
    vectorString: str
    baseScore: int

    accessVector: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    accessComplexity: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    authentication: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    confidentialityImpact: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    integrityImpact: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    availabilityImpact: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    exploitability: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    remediationLevel: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    reportConfidence: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    temporalScore: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    collateralDamagePotential: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    targetDistribution: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    confidentialityRequirement: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    integrityRequirement: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    availabilityRequirement: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    environmentalScore: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )


@dataclass_json
@dataclass
class CVSS3(Validations):
    version: str
    vectorString: str
    baseScore: int
    baseSeverity: str

    attackVector: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    attackComplexity: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    privilegesRequired: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    userInteraction: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    scope: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    confidentialityImpact: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    integrityImpact: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    availabilityImpact: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    exploitCodeMaturity: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    remediationLevel: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    reportConfidence: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    temporalScore: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    temporalSeverity: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    confidentialityRequirement: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    integrityRequirement: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    availabilityRequirement: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    modifiedAttackVector: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    modifiedAttackComplexity: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    modifiedPrivilegesRequired: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    modifiedUserInteraction: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    modifiedScope: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    modifiedConfidentialityImpact: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    modifiedIntegrityImpact: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    modifiedAvailabilityImpact: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    environmentalScore: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    environmentalSeverity: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )

    # temp fixes for msrc report failing to parse due to the fact they are using the wrong format....
    environmentalsScore: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    environmentalsSeverity: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )


@dataclass_json
@dataclass
class CVSS30(CVSS3):
    """i.a.w. https://www.first.org/cvss/cvss-v3.0.json"""

    pass


@dataclass_json
@dataclass
class CVSS31(CVSS3):
    """i.a.w. https://www.first.org/cvss/cvss-v3.1.json"""

    pass


@dataclass_json
@dataclass
class CVSS4(Validations):
    version: str
    vectorString: str
    baseScore: float
    baseSeverity: str

    attackVector: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    attackComplexity: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    attackRequirements: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    privilegesRequired: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    userInteraction: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    vulnConfidentialityImpact: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    vulnIntegrityImpact: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    vulnAvailabilityImpact: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    subConfidentialityImpact: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    subIntegrityImpact: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    subAvailabilityImpact: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    exploitMaturity: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    confidentialityRequirement: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    integrityRequirement: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    availabilityRequirement: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    modifiedAttackVector: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    modifiedAttackComplexity: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    modifiedAttackRequirements: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    modifiedPrivilegesRequired: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    modifiedUserInteraction: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    modifiedVulnConfidentialityImpact: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    modifiedVulnIntegrityImpact: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    modifiedVulnAvailabilityImpact: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    modifiedSubConfidentialityImpact: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    modifiedSubIntegrityImpact: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    modifiedSubAvailabilityImpact: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    Safety: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    Automatable: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    Recovery: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    valueDensity: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    vulnerabilityResponseEffort: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )
    providerUrgency: Optional[str] = field(
        metadata=json_config(exclude=exclude_optional_dict), default=None
    )


@dataclass_json
@dataclass
class CVSS40(CVSS4):
    """i.a.w. https://www.first.org/cvss/cvss-v4.0.json"""

    pass
